(* ---------------------------------------------------------------------- *)
(* Common Object Service 
                                                                          *)
(* ---------------------------------------------------------------------- *)
(* ICosProperty - Interface Module for the Common Object Service
                  "Object Property Service" 

                - Modeled after OMG's Object Property Service 
                  specification.  OMG TC Document 95.6.1
                                                                          *)
(* ---------------------------------------------------------------------- *)

INTERFACE ICosProperty;

(* ---------------------------------------------------------------------- *)
(* some simple types *)

TYPE TString = OPTIONAL ilu.CString;

TYPE TObject = RECORD
                aSBH : TString,
                aType : TString
              END;

(* ---------------------------------------------------------------------- *)
(* The Any type.  ILU should really have this concept.
     Note: this probably should be in another Interface.
*)

TYPE TAnyList = SEQUENCE of Any;

TYPE AnyUnion = UNION
    INTEGER, CARDINAL, BYTE, BOOLEAN, REAL, CHARACTER,
    TObject,
    TAnyList,
    TString
  END;

TYPE Any = OPTIONAL AnyUnion;

(* ---------------------------------------------------------------------- *)
(* property data types *)

TYPE PropertyName = TString;

TYPE Property = RECORD
  property-name : PropertyName,
  property-value : Any
END;

TYPE PropertyNames = SEQUENCE OF PropertyName;
TYPE Properties = SEQUENCE OF Property;

TYPE PropertyValues = SEQUENCE OF Any; (* SWH 7.20.95 *)
TYPE PropertyValuesList = SEQUENCE OF PropertyValues; (* SWH 7.20.95 *)

TYPE PropertiesList = SEQUENCE OF Properties;  (* SWH 6.15.95 *)

(* ---------------------------------------------------------------------- *)
(* exceptions *)

EXCEPTION ConstraintNotSupported;
EXCEPTION InvalidPropertyName;
EXCEPTION ConflictingProperty;
EXCEPTION PropertyNotFound;
EXCEPTION UnsupportedTypeCode;
EXCEPTION UnsupportedProperty;
EXCEPTION UnsupportedMode;
EXCEPTION FixedProperty;
EXCEPTION ReadOnlyProperty;

TYPE ExceptionReason = ENUMERATION 
  invalid-property-name, 
  conflicting-property, 
  property-not-found, 
  unsupported-type-code, 
  unsupported-property, 
  unsupported-mode, 
  fixed-property, 
  read-only-property 
END;

TYPE PropertyException = RECORD
  reason : ExceptionReason,
  failing-property-name : PropertyName
END;

TYPE PropertyExceptions = SEQUENCE OF PropertyException;

TYPE MultipleExceptions = RECORD
  exceptions : PropertyExceptions
END;

EXCEPTION MultipleExceptions : MultipleExceptions;

(* ------------------------------------------------------------ *)

TYPE PropertySet = OBJECT OPTIONAL
  METHODS
    define-property (property-name : PropertyName, 
                     property-value : Any)
      RAISES InvalidPropertyName, ConflictingProperty, UnsupportedTypeCode, 
             UnsupportedProperty, ReadOnlyProperty END,

    define-properties (nproperties : Properties)
      RAISES MultipleExceptions END,

    get-number-of-properties () : CARDINAL,

    get-all-property-names (how-many : CARDINAL, 
                            OUT property-names : PropertyNames, 
                            OUT rest : PropertyNamesIterator),

    get-property-value (property-name : PropertyName) : Any
      RAISES PropertyNotFound, InvalidPropertyName END,

    get-properties (property-names : PropertyNames, 
                    OUT nproperties : Properties) : BOOLEAN,

    get-all-properties (how-many : CARDINAL, 
                        OUT nproperties : Properties, 
                        OUT rest : PropertiesIterator),

    delete-property (property-name : PropertyName)
      RAISES PropertyNotFound, InvalidPropertyName, FixedProperty END,

    delete-properties (property-names : PropertyNames)
      RAISES MultipleExceptions END,

    delete-all-properties () : BOOLEAN,

    is-property-defined (property-name : PropertyName) : BOOLEAN
      RAISES InvalidPropertyName END
  END;

(* ------------------------------------------------------------ *)
(* used with get-all-property-names method *)

TYPE PropertyNamesIterator = OBJECT OPTIONAL
  METHODS
    reset (),
    next-one (OUT property-name : PropertyName) : BOOLEAN,
    next-n (how-many : CARDINAL, OUT property-names : PropertyNames) : BOOLEAN,
    destroy ()
  END;

(* ------------------------------------------------------------ *)
(* used with get-all-properties method *)

TYPE PropertiesIterator = OBJECT OPTIONAL
  METHODS
    reset (),
    next-one (OUT aproperty : Property) : BOOLEAN,
    next-n (how-many : CARDINAL, OUT nproperties : Properties) : BOOLEAN,
    destroy ()
  END;

(* ------------------------------------------------------------ *)
